#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <math.h>
#include <string.h>
#include <stdio.h>

#include "proiect.h"
double a, b;


#define FCC(c1, c2, c3, c4) \
	(((DWORD)(c4) << 24) + \
	 ((DWORD)(c3) << 16) + \
	 (( WORD)(c2) <<  8) + \
	 (( BYTE)(c1) <<  0))


uint8_t	buf[2][256];	// wavie output buffers (double buffering)
const	 uint16_t	buf_size = 256;	// front and back buffer sizes
volatile uint8_t	buf_front = 0;	// front buffer index (current buffer used)
volatile uint8_t	buf_pos = 0;	// current buffer position
volatile uint8_t	buf_sync = 0;

#define BUF_FRONT	(buf[buf_front])
#define BUF_BACK	(buf[1 - buf_front])
FATFS fs;					// sistemul de fisiere

int time;

ISR(TIMER0_COMPA_vect)
{
  if(time == 10) {
      time = 0;
      draw_drumstick(LEFT_X, LEFT_Y, a, DELETE, LEFT_STICK);
      draw_drumstick(RIGHT_X, RIGHT_Y, b, DELETE, RIGHT_STICK);
      drums_init();
      draw_drumstick(LEFT_X, LEFT_Y, gz[LEFT_STICK], DRAW, LEFT_STICK);
      draw_drumstick(RIGHT_X, RIGHT_Y, gz[RIGHT_STICK], DRAW, RIGHT_STICK);
      a = gz[LEFT_STICK];
      b = gz[RIGHT_STICK];

      druum(LEFT_STICK);
      druum(RIGHT_STICK);
  } else {
      time++;
  }

    if(bat == 0)
        mpu_left_getConvData(&axds[bat], &ayds[bat], &azds[bat], &gxds[bat], &gyds[bat], &gzds[bat]);
    else
        mpu_right_getConvData(&axds[bat], &ayds[bat], &azds[bat], &gxds[bat], &gyds[bat], &gzds[bat]);

    if( abs(old_gzds[bat] - gzds[bat]) > LIMIT) {
        gz[bat] += gzds[bat]/10;
        old_gzds[bat] = gzds[bat];

        if(gz[bat] > 360.0)
            gz[bat] -= 360.0;
        if(gz[bat] > 180.0)
            gz[bat] = 180.0;
        if(gz[bat] < 0.0)
            gz[bat] = 0.0;
    }

    if( abs(old_gxds[bat] - gxds[bat]) > 1) {
        gx[bat] += gxds[bat]/10;
        old_gxds[bat] = gxds[bat];

        if(gx[bat] > 360.0)
            gx[bat] -= 360.0;
        if(gx[bat] > 180.0)
            gx[bat] = 180.0;
        if(gx[bat] < 0.0)
            gx[bat] = 0.0;
    }

    bat  = (bat + 1) % 2;

}
ISR(TIMER2_COMPA_vect)
{
	OCR1B = BUF_FRONT[buf_pos++];

	// swap buffers when end is reached (end is 256 <=> overflow to 0)
	if(buf_pos == 0)
		buf_front = 1 - buf_front;
}


void draw_drumstick(int x, int y, float angle, int draw, int stick)
{
    int startX = x, startY = y;
    int stopX, stopY;
    float radians = angle * PI / 180.0f;

		/*  Deseneaza pana la marginea ecranului  */
    if(startX < 0)
        startX = 0;
    if(startX > ST7735R_WIDTH - 1)
        startX = ST7735R_WIDTH - 1;

    if(startY < 0)
        startY = 0;
    if(startY > ST7735R_HEIGHT - 1)
        startY = ST7735R_HEIGHT - 1;

    stopX = startX + (ST7735R_WIDTH - 1) / 2 * sin(radians);
    stopY = startY + (ST7735R_WIDTH - 1) / 2 * cos(radians);

    if(stopX < 0)
        stopX = 0;
    if(stopX > ST7735R_WIDTH - 1)
        stopX = ST7735R_WIDTH - 1;

    if(stopY < 0)
        stopY = 0;
    if(stopY > ST7735R_HEIGHT - 1)
        stopY = ST7735R_HEIGHT - 1;

    sX[stick] = stopX;
    sY[stick] = stopY;

		/*  Deseneaza sau sterge batul  */
    BEGIN_TFT();
    if (draw == DRAW) {
        ST7735R_Line(startX, startY - 1, stopX, stopY - 1, 139, 69, 19);
        ST7735R_Line(startX, startY, stopX, stopY, 139, 69, 19);
        ST7735R_Line(startX, startY + 1, stopX, stopY + 1, 139, 69, 19);
    } else {
        ST7735R_Line(startX, startY - 1, stopX, stopY - 1, 0, 0, 0);
        ST7735R_Line(startX, startY, stopX, stopY, 0, 0, 0);
        ST7735R_Line(startX, startY + 1, stopX, stopY + 1, 0, 0, 0);
    }
    END_TFT();
}

/*  Deseneaza tobele  */
void drums_init(void)
{
    BEGIN_TFT();
    // Extrema stanga
    ST7735R_FilledCircle(drums_x[0], drums_y[0], drum_r, 255, 0, 0);

    // Stanga
    ST7735R_FilledCircle(drums_x[1], drums_y[1], drum_r, 0, 255, 0);


    // Dreapta
    ST7735R_FilledCircle(drums_x[2], drums_y[2], drum_r, 0, 0, 255);

    // Extrema dreapta
    ST7735R_FilledCircle(drums_x[3], drums_y[3], drum_r, 255, 255, 255);

    END_TFT();
}

/*  Calculeaza daca varful batului se afla pe o toba  */
int distance(int circle, int stick)
{
    int x = (drums_x[circle] - sX[stick]) * (drums_x[circle] - sX[stick]);
    int y = (drums_y[circle] - sY[stick]) * (drums_y[circle] - sY[stick]);

    if ( sqrt(x + y) < drum_r )
        return 1;
    return 0;
}

/*  Verifica daca s-a produs o lovitura, generand sunet  */
void druum(int stick)
{
    char text[100];
    strcpy(text, "NONE");

    if(distance(0, stick)) {
        strcpy(text, "cymbal.wav");
    } else if (distance(1, stick)) {
        strcpy(text, "tom.wav");
    } else if (distance(2, stick)) {
        strcpy(text, "tom.wav");
    } else if (distance(3, stick)) {
      strcpy(text, "cymbal.wav");
    }

    if(fabs(gx[stick]) < 0.01){
        if (strcmp(text, "NONE") != 0 && make_it[stick] == 0) {
            END_TFT();
            SD_SELECT();
            play(text);
            SD_DESELECT();
            BEGIN_TFT();
            make_it[stick] = 1;
        }
    } else {
        make_it[stick] = 0;
    }

}

/*  Timer care primeste date de la senzori si deseneaza pe LCD  */
void timer0_init(void)
{
    TCCR0A |= (1 << WGM01) | (1 << WGM00);
    TCCR0B |= (1 << WGM02);
    TCCR0B |= (1 << CS01) | (1 << CS00);
    TIMSK0 = (1 << OCIE0A);
    OCR0A = 100;
}

/*  Timer care genereaza frecventele  */
void timer2_start(void)
{
	// interrupt on compare A
	TIMSK2 |= (1 << OCIE2A);
	// CTC, top OCRA
	TCCR2B |= (0 << WGM22);
	TCCR2A |= (1 << WGM21) | (0 << WGM20);
	// prescaler 8
	TCCR2B |= (2 << CS20);
}

void timer2_stop(void)
{
	TCCR2B = 0;
	TCCR2A = 0;
	TIMSK2 = 0;
	OCR2A = 0;
	TCNT2 = 0;
}

/*  8-bit PWM pentru sunet  */
void timer1_start(void)
{
	// 8-bit FastPWM
	TCCR1B |= (1 << WGM12);
	TCCR1A |= (1 << WGM10);
	// channel B inverted
	TCCR1A |= (1 << COM1B0) | (1 << COM1B1);
	// prescaler 1
	TCCR1B |= (1 << CS10);
}

void timer1_stop(void)
{
	TCCR1B = 0;
	TCCR1A = 0;
	OCR1B = 0;
	TCNT1 = 0;
}

/*  Citire header FAT32  */
DWORD load_header(void)
{
	DWORD size;
	WORD ret;

	// citeste header-ul (12 octeti)
	if(pf_read(BUF_FRONT, 12, &ret))
		return 1;

	if(ret != 12 || LD_DWORD(BUF_FRONT + 8) != FCC('W','A','V','E'))
		return 0;

	for(;;)
	{
		// citeste chunk ID si size
		pf_read(BUF_FRONT, 8, &ret);
		if(ret != 8)
			return 0;

		size = LD_DWORD(&BUF_FRONT[4]);

		// verifica FCC
		switch(LD_DWORD(&BUF_FRONT[0]))
		{
			// 'fmt ' chunk
			case FCC('f','m','t',' '):
				// verifica size
				if(size > 100 || size < 16) return 0;

				// citeste continutul
				pf_read(BUF_FRONT, size, &ret);
				// verifica codificarea
				if(ret != size || BUF_FRONT[0] != 1) return 0;
				// verifica numarul de canale
				if(BUF_FRONT[2] != 1 && BUF_FRONT[2] != 2) return 0;
				// verifica rezolutia
				if(BUF_FRONT[14] != 8 && BUF_FRONT[14] != 16) return 0;

				// seteaza sampling rate-ul
				OCR0A = (BYTE)(F_CPU / 8 / LD_WORD(&BUF_FRONT[4])) - 1;
				break;

			// 'data' chunk => incepe redarea
			case FCC('d','a','t','a'):
				return size;

			// 'LIST' chunk => skip
			case FCC('L','I','S','T'):
			// 'fact' chunk => skip
			case FCC('f','a','c','t'):
				pf_lseek(fs.fptr + size);
				break;

			// chunk necunoscut => eroare
			default:
				return 0;
		}
	}

	return 0;
}

/*  Genereaza un sunet de pe cardul microSD  */
UINT play(const char *path)
{
	FRESULT ret;

	if((ret = pf_open(path)) == FR_OK)
	{
		WORD bytes_read;

		// incarca header-ul fisierului
		DWORD current_size = load_header();
		if(current_size < buf_size)
			return FR_NO_FILE;

		// align to sector boundary
		ret = pf_lseek((fs.fptr + 511) & ~511);
		if(ret != FR_OK)
			return ret;

		// fill front buffer
		ret = pf_read(BUF_FRONT, buf_size, &bytes_read);
		if(ret != FR_OK)
			return ret;
		if(bytes_read < buf_size)
			return ret;

		// reset front buffer index
		buf_pos = 0;


		// start output
		timer2_start();
		timer1_start();
		DDRD |= (1 << PD4);

		while(1)
		{
			uint8_t old_buf_front = buf_front;

			// fill back buffer
			ret = pf_read(BUF_BACK, buf_size, &bytes_read);
			if(ret != FR_OK)
				break;
			if(bytes_read < buf_size)
				break;

			// wait for buffer swap
			while(old_buf_front == buf_front) ;
		}

		// stop output
		DDRD &= ~(1 << PD4);
		timer1_stop();
		timer2_stop();
	}


	return ret;
}

int main(void)
{
    /*  Led pentru debugging  */
    DDRD |= (1 << PD7);

		/*  Porneste senzorii de pe bete  */
    mpu_left_init();
    mpu_right_init();

		/*  Pornire intreruperi  */
    sei();

		/*  Initializare card microSD  */
    SD_SELECT();
     while(1) {
       if(pf_mount(&fs) != FR_OK) {
         PORTD |= (1 << PD7);
         _delay_ms(500);
         PORTD &= ~(1 << PD7);
         _delay_ms(500);
 				continue;
 			} else {
         PORTD |= (1 << PD7);
        break;
      }
     }
    SD_DESELECT();

		/*  Initializare LCD  */
    ST7735R_Begin();
    ST7735R_FillRect(0, 0, ST7735R_WIDTH - 1, ST7735R_HEIGHT - 1, 0, 0, 0);
    drums_init();

		/*  incepe ascultarea senzorilor  */
    timer0_init();

    while(1);

    return 0;
}
